/*
 * Copyright (C) 2008 Emweb bvba, Heverlee, Belgium.
 *
 * See the LICENSE file for terms of use.
 */

#include <Wt/WApplication>
#include <Wt/WBreak>
#include <Wt/WContainerWidget>
#include <Wt/WLineEdit>
#include <Wt/WPushButton>
#include <Wt/WTableView>
#include <Wt/WText>
#include <memory>
#include "VirtualModel.h"

// c++0x only, for std::bind
// #include <functional>

using namespace Wt;

/*
 * A simple hello world application class which demonstrates how to react
 * to events, read input, and give feed-back.
 */
class HelloApplication : public WApplication
{
public:
  HelloApplication(const WEnvironment& env);

};

/*
 * The env argument contains information about the new session, and
 * the initial request. It must be passed to the WApplication
 * constructor so it is typically also an argument for your custom
 * application constructor.
*/
HelloApplication::HelloApplication(const WEnvironment& env)
  : WApplication(env)
{
  setTitle("Hello world");                               // application title


  auto tableView = std::unique_ptr<WTableView>(new Wt::WTableView());
  tableView->setModel(new VirtualModel(10000, 50, tableView.get()));

  tableView->setRowHeaderCount(1); // treat first column as 'fixed' row headers
  tableView->setSortingEnabled(false);
  tableView->setAlternatingRowColors(true);
  tableView->setRowHeight(60); //<- this work
  //tableView->setRowHeight(Wt::WLength(1, Wt::WLength::Centimeter)); // <- this won't work after scrolling down.
  tableView->setHeaderHeight(28);
  tableView->setSelectionMode(Wt::ExtendedSelection);
  tableView->setEditTriggers(Wt::WAbstractItemView::NoEditTrigger);

  tableView->resize(650, 400);
  root()->addWidget(tableView.release());


  auto tableView2 = std::unique_ptr<WTableView>(new Wt::WTableView());
  tableView2->setModel(new VirtualModel(10000, 50, tableView2.get()));

  tableView2->setRowHeaderCount(1); // treat first column as 'fixed' row headers
  tableView2->setSortingEnabled(false);
  tableView2->setAlternatingRowColors(true);
  //tableView2->setRowHeight(28); //<- this work
  tableView2->setRowHeight(Wt::WLength(1, Wt::WLength::Centimeter)); // <- this won't work after scrolling down for while.
  tableView2->setHeaderHeight(28);
  tableView2->setSelectionMode(Wt::ExtendedSelection);
  tableView2->setEditTriggers(Wt::WAbstractItemView::NoEditTrigger);

  tableView2->resize(650, 400);
  root()->addWidget(tableView2.release());
}

WApplication *createApplication(const WEnvironment& env)
{
  /*
   * You could read information from the environment to decide whether
   * the user has permission to start a new application
   */
  return new HelloApplication(env);
}

int main(int argc, char **argv)
{
  /*
   * Your main method may set up some shared resources, but should then
   * start the server application (FastCGI or httpd) that starts listening
   * for requests, and handles all of the application life cycles.
   *
   * The last argument to WRun specifies the function that will instantiate
   * new application objects. That function is executed when a new user surfs
   * to the Wt application, and after the library has negotiated browser
   * support. The function should return a newly instantiated application
   * object.
   */
  return WRun(argc, argv, &createApplication);
}

